@@ -1,40 +1,37 @@
-"""
-Copyright  2014-2021 Vincent Texier <vit@free.fr>
-
-DuniterPy is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-DuniterPy is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# Copyright  2014-2022 Vincent Texier <vit@free.fr>
+#
+# DuniterPy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# DuniterPy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import re
-from typing import Any, Optional, TypeVar, Type, Dict
+from ipaddress import ip_address
+from typing import Any, Dict, Optional, Tuple, Type, TypeVar
 
-from aiohttp import ClientSession
+from duniterpy import constants as const
 
-import duniterpy.constants as constants
 from ..documents import MalformedDocumentError
 
 
 class ConnectionHandler:
-    """Helper class used by other API classes to ease passing server connection information."""
+    """Helper class used by other API classes to ease passing address connection information."""
 
     def __init__(
         self,
         http_scheme: str,
         ws_scheme: str,
-        server: str,
+        address: str,
         port: int,
         path: str,
-        session: ClientSession,
         proxy: Optional[str] = None,
     ) -> None:
         """
@@ -42,22 +39,20 @@ class ConnectionHandler:
 
         :param http_scheme: Http scheme
         :param ws_scheme: Web socket scheme
-        :param server: Server IP or domain name
+        :param address: Domain name, IPv6, or IPv4 address
         :param port: Port number
         :param port: Url path
-        :param session: Session AIOHTTP
         :param proxy: Proxy (optional, default=None)
         """
         self.http_scheme = http_scheme
         self.ws_scheme = ws_scheme
-        self.server = server
+        self.address = address
         self.port = port
         self.path = path
         self.proxy = proxy
-        self.session = session
 
     def __str__(self) -> str:
-        return "connection info: %s:%d" % (self.server, self.port)
+        return f"connection info: {self.address}:{self.port}"
 
 
 # required to type hint cls in classmethod
@@ -72,9 +67,7 @@ class Endpoint:
     def inline(self) -> str:
         raise NotImplementedError("inline() is not implemented")
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         raise NotImplementedError("conn_handler is not implemented")
 
     def __str__(self) -> str:
@@ -118,25 +111,21 @@ class UnknownEndpoint(Endpoint):
         """
         doc = self.api
         for p in self.properties:
-            doc += " {0}".format(p)
+            doc += f" {p}"
         return doc
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
-        Return connection handler from session
+        Return connection handler
 
-        :param session: AIOHTTP Session
-        :param proxy: Proxy server
+        :param proxy: Proxy address
         :return:
         """
-        return ConnectionHandler("", "", "", 0, "", ClientSession())
+        return ConnectionHandler("", "", "", 0, "")
 
     def __str__(self) -> str:
-        return "{0} {1}".format(
-            self.api, " ".join(["{0}".format(p) for p in self.properties])
-        )
+        properties = " ".join([f"{p}" for p in self.properties])
+        return f"{self.api} {properties}"
 
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, UnknownEndpoint):
@@ -154,23 +143,19 @@ BMAEndpointType = TypeVar("BMAEndpointType", bound="BMAEndpoint")
 class BMAEndpoint(Endpoint):
     API = "BASIC_MERKLED_API"
     re_inline = re.compile(
-        "^BASIC_MERKLED_API(?: ({host_regex}))?(?: ({ipv4_regex}))?(?: ({ipv6_regex}))?(?: ([0-9]+))$".format(
-            host_regex=constants.HOST_REGEX,
-            ipv4_regex=constants.IPV4_REGEX,
-            ipv6_regex=constants.IPV6_REGEX,
-        )
+        f"^{API}(?: (?P<host>{const.HOST_REGEX}))?(?: (?P<ipv4>{const.IPV4_REGEX}))?(?: (?P<ipv6>{const.IPV6_REGEX}))?(?: (?P<port>{const.PORT_REGEX}))$"
     )
 
-    def __init__(self, server: str, ipv4: str, ipv6: str, port: int) -> None:
+    def __init__(self, host: str, ipv4: str, ipv6: str, port: int) -> None:
         """
         Init BMAEndpoint instance
 
-        :param server: IP or domain name
+        :param host: Hostname
         :param ipv4: IP as IPv4 format
         :param ipv6: IP as IPv6 format
         :param port: Port number
         """
-        self.server = server
+        self.host = host
         self.ipv4 = ipv4
         self.ipv6 = ipv6
         self.port = port
@@ -186,11 +171,11 @@ class BMAEndpoint(Endpoint):
         m = BMAEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(BMAEndpoint.API)
-        server = m.group(1)
-        ipv4 = m.group(2)
-        ipv6 = m.group(3)
-        port = int(m.group(4))
-        return cls(server, ipv4, ipv6, port)
+        host, ipv4 = fix_host_ipv4_mix_up(m["host"], m["ipv4"])
+        ipv6 = m["ipv6"]
+        port = int(m["port"])
+
+        return cls(host, ipv4, ipv6, port)
 
     def inline(self) -> str:
         """
@@ -198,34 +183,29 @@ class BMAEndpoint(Endpoint):
 
         :return:
         """
-        return BMAEndpoint.API + "{DNS}{IPv4}{IPv6}{PORT}".format(
-            DNS=(" {0}".format(self.server) if self.server else ""),
-            IPv4=(" {0}".format(self.ipv4) if self.ipv4 else ""),
-            IPv6=(" {0}".format(self.ipv6) if self.ipv6 else ""),
-            PORT=(" {0}".format(self.port) if self.port else ""),
-        )
+        inlined = [
+            str(info) for info in (self.host, self.ipv4, self.ipv6, self.port) if info
+        ]
+        return f'{self.API} {" ".join(inlined)}'
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        if self.server:
+        if self.host:
             conn_handler = ConnectionHandler(
-                "http", "ws", self.server, self.port, "", session, proxy
+                "http", "ws", self.host, self.port, "", proxy
             )
         elif self.ipv6:
             conn_handler = ConnectionHandler(
-                "http", "ws", "[{0}]".format(self.ipv6), self.port, "", session, proxy
+                "http", "ws", f"[{self.ipv6}]", self.port, "", proxy
             )
         else:
             conn_handler = ConnectionHandler(
-                "http", "ws", self.ipv4, self.port, "", session, proxy
+                "http", "ws", self.ipv4, self.port, "", proxy
             )
 
         return conn_handler
@@ -237,14 +217,14 @@ class BMAEndpoint(Endpoint):
         if not isinstance(other, BMAEndpoint):
             return NotImplemented
         return (
-            self.server == other.server
+            self.host == other.host
             and self.ipv4 == other.ipv4
             and self.ipv6 == other.ipv6
             and self.port == other.port
         )
 
     def __hash__(self) -> int:
-        return hash((self.server, self.ipv4, self.ipv6, self.port))
+        return hash((self.host, self.ipv4, self.ipv6, self.port))
 
 
 # required to type hint cls in classmethod
@@ -254,25 +234,20 @@ SecuredBMAEndpointType = TypeVar("SecuredBMAEndpointType", bound="SecuredBMAEndp
 class SecuredBMAEndpoint(BMAEndpoint):
     API = "BMAS"
     re_inline = re.compile(
-        "^BMAS(?: ({host_regex}))?(?: ({ipv4_regex}))?(?: ({ipv6_regex}))? ([0-9]+)(?: ({path_regex}))?$".format(
-            host_regex=constants.HOST_REGEX,
-            ipv4_regex=constants.IPV4_REGEX,
-            ipv6_regex=constants.IPV6_REGEX,
-            path_regex=constants.PATH_REGEX,
-        )
+        f"^{API}(?: (?P<host>{const.HOST_REGEX}))?(?: (?P<ipv4>{const.IPV4_REGEX}))?(?: (?P<ipv6>{const.IPV6_REGEX}))? (?P<port>{const.PORT_REGEX})(?: (?P<path>{const.PATH_REGEX}))?$"
     )
 
-    def __init__(self, server: str, ipv4: str, ipv6: str, port: int, path: str) -> None:
+    def __init__(self, host: str, ipv4: str, ipv6: str, port: int, path: str) -> None:
         """
         Init SecuredBMAEndpoint instance
 
-        :param server: IP or domain name
+        :param host: Hostname
         :param ipv4: IP as IPv4 format
         :param ipv6: IP as IPv6 format
         :param port: Port number
         :param path: Url path
         """
-        super().__init__(server, ipv4, ipv6, port)
+        super().__init__(host, ipv4, ipv6, port)
         self.path = path
 
     @classmethod
@@ -288,14 +263,14 @@ class SecuredBMAEndpoint(BMAEndpoint):
         m = SecuredBMAEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(SecuredBMAEndpoint.API)
-        server = m.group(1)
-        ipv4 = m.group(2)
-        ipv6 = m.group(3)
-        port = int(m.group(4))
-        path = m.group(5)
+        host, ipv4 = fix_host_ipv4_mix_up(m["host"], m["ipv4"])
+        ipv6 = m["ipv6"]
+        port = int(m["port"])
+        path = m["path"]
+
         if not path:
             path = ""
-        return cls(server, ipv4, ipv6, port, path)
+        return cls(host, ipv4, ipv6, port, path)
 
     def inline(self) -> str:
         """
@@ -305,38 +280,29 @@ class SecuredBMAEndpoint(BMAEndpoint):
         """
         inlined = [
             str(info)
-            for info in (self.server, self.ipv4, self.ipv6, self.port, self.path)
+            for info in (self.host, self.ipv4, self.ipv6, self.port, self.path)
             if info
         ]
-        return SecuredBMAEndpoint.API + " " + " ".join(inlined)
+        return f'{self.API} {" ".join(inlined)}'
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        if self.server:
+        if self.host:
             conn_handler = ConnectionHandler(
-                "https", "wss", self.server, self.port, self.path, session, proxy
+                "https", "wss", self.host, self.port, self.path, proxy
             )
         elif self.ipv6:
             conn_handler = ConnectionHandler(
-                "https",
-                "wss",
-                "[{0}]".format(self.ipv6),
-                self.port,
-                self.path,
-                session,
-                proxy,
+                "https", "wss", f"[{self.ipv6}]", self.port, self.path, proxy
             )
         else:
             conn_handler = ConnectionHandler(
-                "https", "wss", self.ipv4, self.port, self.path, session, proxy
+                "https", "wss", self.ipv4, self.port, self.path, proxy
             )
 
         return conn_handler
@@ -349,18 +315,12 @@ WS2PEndpointType = TypeVar("WS2PEndpointType", bound="WS2PEndpoint")
 class WS2PEndpoint(Endpoint):
     API = "WS2P"
     re_inline = re.compile(
-        "^WS2P ({ws2pid_regex}) ((?:{host_regex})|(?:{ipv4_regex})|(?:{ipv6_regex})) ([0-9]+)?(?: ({path_regex}))?$".format(
-            ws2pid_regex=constants.WS2PID_REGEX,
-            host_regex=constants.HOST_REGEX,
-            ipv4_regex=constants.IPV4_REGEX,
-            ipv6_regex=constants.IPV6_REGEX,
-            path_regex=constants.PATH_REGEX,
-        )
+        f"^{API} (?P<ws2pid>{const.WS2PID_REGEX}) (?P<host>(?:{const.HOST_REGEX})|(?:{const.IPV4_REGEX})|(?:{const.IPV6_REGEX})) (?P<port>{const.PORT_REGEX})?(?: (?P<path>{const.PATH_REGEX}))?$"
     )
 
-    def __init__(self, ws2pid: str, server: str, port: int, path: str) -> None:
+    def __init__(self, ws2pid: str, host: str, port: int, path: str) -> None:
         self.ws2pid = ws2pid
-        self.server = server
+        self.host = host
         self.port = port
         self.path = path
 
@@ -375,13 +335,13 @@ class WS2PEndpoint(Endpoint):
         m = WS2PEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(WS2PEndpoint.API)
-        ws2pid = m.group(1)
-        server = m.group(2)
-        port = int(m.group(3))
-        path = m.group(4)
+        ws2pid = m["ws2pid"]
+        host = m["host"]
+        port = int(m["port"])
+        path = m["path"]
         if not path:
             path = ""
-        return cls(ws2pid, server, port, path)
+        return cls(ws2pid, host, port, path)
 
     def inline(self) -> str:
         """
@@ -390,19 +350,14 @@ class WS2PEndpoint(Endpoint):
         :return:
         """
         inlined = [
-            str(info)
-            for info in (self.ws2pid, self.server, self.port, self.path)
-            if info
+            str(info) for info in (self.ws2pid, self.host, self.port, self.path) if info
         ]
-        return WS2PEndpoint.API + " " + " ".join(inlined)
+        return f'{self.API} {" ".join(inlined)}'
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
@@ -412,13 +367,7 @@ class WS2PEndpoint(Endpoint):
             http_scheme += "s"
             websocket_scheme += "s"
         return ConnectionHandler(
-            http_scheme,
-            websocket_scheme,
-            self.server,
-            self.port,
-            self.path,
-            session,
-            proxy,
+            http_scheme, websocket_scheme, self.host, self.port, self.path, proxy
         )
 
     def __str__(self) -> str:
@@ -428,14 +377,14 @@ class WS2PEndpoint(Endpoint):
         if not isinstance(other, WS2PEndpoint):
             return NotImplemented
         return (
-            self.server == other.server
+            self.host == other.host
             and self.ws2pid == other.ws2pid
             and self.port == other.port
             and self.path == other.path
         )
 
     def __hash__(self) -> int:
-        return hash((self.ws2pid, self.server, self.port, self.path))
+        return hash((self.ws2pid, self.host, self.port, self.path))
 
 
 # required to type hint cls in classmethod
@@ -445,13 +394,11 @@ ESCoreEndpointType = TypeVar("ESCoreEndpointType", bound="ESCoreEndpoint")
 class ESCoreEndpoint(Endpoint):
     API = "ES_CORE_API"
     re_inline = re.compile(
-        "^ES_CORE_API ((?:{host_regex})|(?:{ipv4_regex})) ([0-9]+)$".format(
-            host_regex=constants.HOST_REGEX, ipv4_regex=constants.IPV4_REGEX
-        )
+        f"^{API} (?P<host>(?:{const.HOST_REGEX})|(?:{const.IPV4_REGEX})) (?P<port>{const.PORT_REGEX})$"
     )
 
-    def __init__(self, server: str, port: int) -> None:
-        self.server = server
+    def __init__(self, host: str, port: int) -> None:
+        self.host = host
         self.port = port
 
     @classmethod
@@ -465,9 +412,9 @@ class ESCoreEndpoint(Endpoint):
         m = ESCoreEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(ESCoreEndpoint.API)
-        server = m.group(1)
-        port = int(m.group(2))
-        return cls(server, port)
+        host = m["host"]
+        port = int(m["port"])
+        return cls(host, port)
 
     def inline(self) -> str:
         """
@@ -475,22 +422,17 @@ class ESCoreEndpoint(Endpoint):
 
         :return:
         """
-        inlined = [str(info) for info in (self.server, self.port) if info]
-        return ESCoreEndpoint.API + " " + " ".join(inlined)
+        inlined = [str(info) for info in (self.host, self.port) if info]
+        return f'{self.API} {" ".join(inlined)}'
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        return ConnectionHandler(
-            "https", "wss", self.server, self.port, "", session, proxy
-        )
+        return ConnectionHandler("https", "wss", self.host, self.port, "", proxy)
 
     def __str__(self) -> str:
         return self.inline()
@@ -498,10 +440,10 @@ class ESCoreEndpoint(Endpoint):
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, ESCoreEndpoint):
             return NotImplemented
-        return self.server == other.server and self.port == other.port
+        return self.host == other.host and self.port == other.port
 
     def __hash__(self) -> int:
-        return hash((self.server, self.port))
+        return hash((self.host, self.port))
 
 
 # required to type hint cls in classmethod
@@ -511,13 +453,11 @@ ESUserEndpointType = TypeVar("ESUserEndpointType", bound="ESUserEndpoint")
 class ESUserEndpoint(Endpoint):
     API = "ES_USER_API"
     re_inline = re.compile(
-        "^ES_USER_API ((?:{host_regex})|(?:{ipv4_regex})) ([0-9]+)$".format(
-            host_regex=constants.HOST_REGEX, ipv4_regex=constants.IPV4_REGEX
-        )
+        f"^{API} (?P<host>(?:{const.HOST_REGEX})|(?:{const.IPV4_REGEX})) (?P<port>{const.PORT_REGEX})$"
     )
 
-    def __init__(self, server: str, port: int) -> None:
-        self.server = server
+    def __init__(self, host: str, port: int) -> None:
+        self.host = host
         self.port = port
 
     @classmethod
@@ -531,9 +471,9 @@ class ESUserEndpoint(Endpoint):
         m = ESUserEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(ESUserEndpoint.API)
-        server = m.group(1)
-        port = int(m.group(2))
-        return cls(server, port)
+        host = m["host"]
+        port = int(m["port"])
+        return cls(host, port)
 
     def inline(self) -> str:
         """
@@ -541,22 +481,17 @@ class ESUserEndpoint(Endpoint):
 
         :return:
         """
-        inlined = [str(info) for info in (self.server, self.port) if info]
-        return ESUserEndpoint.API + " " + " ".join(inlined)
+        inlined = [str(info) for info in (self.host, self.port) if info]
+        return f'{self.API} {" ".join(inlined)}'
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        return ConnectionHandler(
-            "https", "wss", self.server, self.port, "", session, proxy
-        )
+        return ConnectionHandler("https", "wss", self.host, self.port, "", proxy)
 
     def __str__(self) -> str:
         return self.inline()
@@ -564,10 +499,10 @@ class ESUserEndpoint(Endpoint):
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, ESUserEndpoint):
             return NotImplemented
-        return self.server == other.server and self.port == other.port
+        return self.host == other.host and self.port == other.port
 
     def __hash__(self) -> int:
-        return hash((self.server, self.port))
+        return hash((self.host, self.port))
 
 
 # required to type hint cls in classmethod
@@ -579,13 +514,11 @@ ESSubscribtionEndpointType = TypeVar(
 class ESSubscribtionEndpoint(Endpoint):
     API = "ES_SUBSCRIPTION_API"
     re_inline = re.compile(
-        "^ES_SUBSCRIPTION_API ((?:{host_regex})|(?:{ipv4_regex})) ([0-9]+)$".format(
-            host_regex=constants.HOST_REGEX, ipv4_regex=constants.IPV4_REGEX
-        )
+        f"^{API} (?P<host>(?:{const.HOST_REGEX})|(?:{const.IPV4_REGEX})) (?P<port>{const.PORT_REGEX})$"
     )
 
-    def __init__(self, server: str, port: int) -> None:
-        self.server = server
+    def __init__(self, host: str, port: int) -> None:
+        self.host = host
         self.port = port
 
     @classmethod
@@ -601,9 +534,9 @@ class ESSubscribtionEndpoint(Endpoint):
         m = ESSubscribtionEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(ESSubscribtionEndpoint.API)
-        server = m.group(1)
-        port = int(m.group(2))
-        return cls(server, port)
+        host = m["host"]
+        port = int(m["port"])
+        return cls(host, port)
 
     def inline(self) -> str:
         """
@@ -611,22 +544,17 @@ class ESSubscribtionEndpoint(Endpoint):
 
         :return:
         """
-        inlined = [str(info) for info in (self.server, self.port) if info]
-        return ESSubscribtionEndpoint.API + " " + " ".join(inlined)
+        inlined = [str(info) for info in (self.host, self.port) if info]
+        return f'{self.API} {" ".join(inlined)}'
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        return ConnectionHandler(
-            "https", "wss", self.server, self.port, "", session, proxy
-        )
+        return ConnectionHandler("https", "wss", self.host, self.port, "", proxy)
 
     def __str__(self) -> str:
         return self.inline()
@@ -634,10 +562,10 @@ class ESSubscribtionEndpoint(Endpoint):
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, ESSubscribtionEndpoint):
             return NotImplemented
-        return self.server == other.server and self.port == other.port
+        return self.host == other.host and self.port == other.port
 
     def __hash__(self) -> int:
-        return hash((ESSubscribtionEndpoint.API, self.server, self.port))
+        return hash((ESSubscribtionEndpoint.API, self.host, self.port))
 
 
 # required to type hint cls in classmethod
@@ -646,13 +574,13 @@ GVAEndpointType = TypeVar("GVAEndpointType", bound="GVAEndpoint")
 
 class GVAEndpoint(Endpoint):
     API = "GVA"
-    endpoint_format = f"^GVA(?: ({constants.ENDPOINT_FLAGS_REGEX}))?(?: ({constants.HOST_REGEX}))?(?: ({constants.IPV4_REGEX}))?(?: ({constants.IPV6_REGEX}))? ([0-9]+)(?: ({constants.PATH_REGEX}))?$"
+    endpoint_format = f"^GVA(?: (?P<flags>{const.ENDPOINT_FLAGS_REGEX}))?(?: (?P<host>{const.HOST_REGEX}))?(?: (?P<ipv4>{const.IPV4_REGEX}))?(?: (?P<ipv6>{const.IPV6_REGEX}))? (?P<port>{const.PORT_REGEX})(?: (?P<path>{const.PATH_REGEX}))?$"
     re_inline = re.compile(endpoint_format)
 
     def __init__(
         self,
         flags: str,
-        server: str,
+        host: str,
         ipv4: str,
         ipv6: str,
         port: int,
@@ -662,14 +590,14 @@ class GVAEndpoint(Endpoint):
         Init GVAEndpoint instance
 
         :param flags: Flags of endpoint
-        :param server: IP or domain name
+        :param host: Hostname
         :param ipv4: IP as IPv4 format
         :param ipv6: IP as IPv6 format
         :param port: Port number
         :param path: Url path
         """
         self.flags = flags
-        self.server = server
+        self.host = host
         self.ipv4 = ipv4
         self.ipv6 = ipv6
         self.port = port
@@ -686,17 +614,17 @@ class GVAEndpoint(Endpoint):
         m = cls.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(cls.API)
-        flags = m.group(1)
-        server = m.group(2)
-        ipv4 = m.group(3)
-        ipv6 = m.group(4)
-        port = int(m.group(5))
-        path = m.group(6)
+        flags = m["flags"]
+        host, ipv4 = fix_host_ipv4_mix_up(m["host"], m["ipv4"])
+        ipv6 = m["ipv6"]
+        port = int(m["port"])
+        path = m["path"]
+
         if not flags:
             flags = ""
         if not path:
             path = ""
-        return cls(flags, server, ipv4, ipv6, port, path)
+        return cls(flags, host, ipv4, ipv6, port, path)
 
     def inline(self) -> str:
         """
@@ -708,7 +636,7 @@ class GVAEndpoint(Endpoint):
             str(info)
             for info in (
                 self.flags,
-                self.server,
+                self.host,
                 self.ipv4,
                 self.ipv6,
                 self.port,
@@ -716,44 +644,34 @@ class GVAEndpoint(Endpoint):
             )
             if info
         ]
-        return self.API + " " + " ".join(inlined)
+        return f'{self.API} {" ".join(inlined)}'
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
         scheme_http = "https" if "S" in self.flags else "http"
         scheme_ws = "wss" if "S" in self.flags else "ws"
 
-        if self.server:
+        if self.host:
             conn_handler = ConnectionHandler(
-                scheme_http,
-                scheme_ws,
-                self.server,
-                self.port,
-                self.path,
-                session,
-                proxy,
+                scheme_http, scheme_ws, self.host, self.port, self.path, proxy
             )
         elif self.ipv6:
             conn_handler = ConnectionHandler(
                 scheme_http,
                 scheme_ws,
-                "[{0}]".format(self.ipv6),
+                f"[{self.ipv6}]",
                 self.port,
                 self.path,
-                session,
                 proxy,
             )
         else:
             conn_handler = ConnectionHandler(
-                scheme_http, scheme_ws, self.ipv4, self.port, self.path, session, proxy
+                scheme_http, scheme_ws, self.ipv4, self.port, self.path, proxy
             )
 
         return conn_handler
@@ -765,24 +683,16 @@ class GVAEndpoint(Endpoint):
         if not isinstance(other, self.__class__):
             return NotImplemented
         return (
-            self.server == other.server
+            self.flags == other.flags
+            and self.host == other.host
             and self.ipv4 == other.ipv4
             and self.ipv6 == other.ipv6
             and self.port == other.port
+            and self.path == other.path
         )
 
     def __hash__(self) -> int:
-        return hash((self.server, self.ipv4, self.ipv6, self.port))
-
-
-# required to type hint cls in classmethod
-GVASUBEndpointType = TypeVar("GVASUBEndpointType", bound="GVASUBEndpoint")
-
-
-class GVASUBEndpoint(GVAEndpoint):
-    API = "GVASUB"
-    endpoint_format = f"^GVASUB(?: ({constants.ENDPOINT_FLAGS_REGEX}))?(?: ({constants.HOST_REGEX}))?(?: ({constants.IPV4_REGEX}))?(?: ({constants.IPV6_REGEX}))? ([0-9]+)(?: ({constants.PATH_REGEX}))?$"
-    re_inline = re.compile(endpoint_format)
+        return hash((self.flags, self.host, self.ipv4, self.ipv6, self.port, self.path))
 
 
 MANAGED_API = {
@@ -793,7 +703,6 @@ MANAGED_API = {
     ESUserEndpoint.API: ESUserEndpoint,
     ESSubscribtionEndpoint.API: ESSubscribtionEndpoint,
     GVAEndpoint.API: GVAEndpoint,
-    GVASUBEndpoint.API: GVASUBEndpoint,
 }  # type: Dict[str, Any]
 
 
@@ -812,9 +721,18 @@ def endpoint(value: Any) -> Any:
     elif isinstance(value, str):
         # find Endpoint instance
         for api, cls in MANAGED_API.items():
-            if value.startswith(api + " "):
+            if value.startswith(f"{api} "):
                 result = cls.from_inline(value)
     else:
-        raise TypeError("Cannot convert {0} to endpoint".format(value))
+        raise TypeError(f"Cannot convert {value} to endpoint")
 
     return result
+
+
+def fix_host_ipv4_mix_up(host: str, ipv4: str) -> Tuple[str, str]:
+    mixed_up = False
+    try:
+        mixed_up = ip_address(host).version == 4 and not ipv4
+    except ValueError:
+        pass
+    return ("", host) if mixed_up else (host, ipv4)
