@@ -1,37 +1,39 @@
-"""
-Copyright  2014-2021 Vincent Texier <vit@free.fr>
-
-DuniterPy is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-DuniterPy is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# Copyright  2014-2022 Vincent Texier <vit@free.fr>
+#
+# DuniterPy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# DuniterPy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import base64
 import hashlib
 import re
-from typing import TypeVar, Type, Optional, List, Sequence
-from .block_uid import BlockUID
-from .certification import Certification
-from .revocation import Revocation
-from .identity import Identity
-from .document import Document, MalformedDocumentError
-from .membership import Membership
-from .transaction import Transaction
-from ..constants import PUBKEY_REGEX, BLOCK_HASH_REGEX
+from typing import List, Optional, Sequence, Type, TypeVar
 
+from ..constants import BLOCK_HASH_REGEX, G1_CURRENCY_CODENAME, PUBKEY_REGEX
 
 # required to type hint cls in classmethod
+from ..key import SigningKey, VerifyingKey
+from .block_id import BlockID
+from .certification import Certification
+from .document import Document, MalformedDocumentError
+from .identity import Identity
+from .membership import Membership
+from .revocation import Revocation
+from .transaction import Transaction
+
 BlockType = TypeVar("BlockType", bound="Block")
 
+VERSION = 12
+
 
 class Block(Document):
     """
@@ -85,18 +87,12 @@ class Block(Document):
     re_mediantime = re.compile("MedianTime: ([0-9]+)\n")
     re_universaldividend = re.compile("UniversalDividend: ([0-9]+)\n")
     re_unitbase = re.compile("UnitBase: ([0-9]+)\n")
-    re_issuer = re.compile(
-        "Issuer: ({pubkey_regex})\n".format(pubkey_regex=PUBKEY_REGEX)
-    )
+    re_issuer = re.compile(f"Issuer: ({PUBKEY_REGEX})\n")
     re_issuers_frame = re.compile("IssuersFrame: ([0-9]+)\n")
     re_issuers_frame_var = re.compile("IssuersFrameVar: (0|-?[1-9]\\d{0,18})\n")
     re_different_issuers_count = re.compile("DifferentIssuersCount: ([0-9]+)\n")
-    re_previoushash = re.compile(
-        "PreviousHash: ({block_hash_regex})\n".format(block_hash_regex=BLOCK_HASH_REGEX)
-    )
-    re_previousissuer = re.compile(
-        "PreviousIssuer: ({pubkey_regex})\n".format(pubkey_regex=PUBKEY_REGEX)
-    )
+    re_previoushash = re.compile(f"PreviousHash: ({BLOCK_HASH_REGEX})\n")
+    re_previousissuer = re.compile(f"PreviousIssuer: ({PUBKEY_REGEX})\n")
     re_parameters = re.compile(
         "Parameters: ([0-9]+\\.[0-9]+):([0-9]+):([0-9]+):([0-9]+):([0-9]+):([0-9]+):\
 ([0-9]+):([0-9]+):([0-9]+):([0-9]+):([0-9]+\\.[0-9]+):([0-9]+):([0-9]+):([0-9]+):([0-9]+):([0-9]+):([0-9]+\\.[0-9]+):\
@@ -109,12 +105,10 @@ class Block(Document):
     re_leavers = re.compile("Leavers:\n")
     re_revoked = re.compile("Revoked:\n")
     re_excluded = re.compile("Excluded:\n")
-    re_exclusion = re.compile("({pubkey_regex})\n".format(pubkey_regex=PUBKEY_REGEX))
+    re_exclusion = re.compile(f"({PUBKEY_REGEX})\n")
     re_certifications = re.compile("Certifications:\n")
     re_transactions = re.compile("Transactions:\n")
-    re_hash = re.compile(
-        "InnerHash: ({block_hash_regex})\n".format(block_hash_regex=BLOCK_HASH_REGEX)
-    )
+    re_hash = re.compile(f"InnerHash: ({BLOCK_HASH_REGEX})\n")
     re_nonce = re.compile("Nonce: ([0-9]+)\n")
 
     fields_parsers = {
@@ -150,8 +144,6 @@ class Block(Document):
 
     def __init__(
         self,
-        version: int,
-        currency: str,
         number: int,
         powmin: int,
         time: int,
@@ -176,13 +168,13 @@ class Block(Document):
         transactions: List[Transaction],
         inner_hash: str,
         nonce: int,
-        signature: str,
+        signing_key: SigningKey = None,
+        version: int = VERSION,
+        currency: str = G1_CURRENCY_CODENAME,
     ) -> None:
         """
         Constructor
 
-        :param version: duniter protocol version
-        :param currency: the block currency
         :param number: the number of the block
         :param powmin: the powmin value of this block
         :param time: the timestamp of this block
@@ -207,9 +199,12 @@ class Block(Document):
         :param transactions: transactions documents
         :param inner_hash: the block hash
         :param nonce: the nonce value of the block
-        :param signature: the block signature
+        :param signing_key: SigningKey instance to sign the document (default=None)
+        :param version: Document version (default=block.VERSION)
+        :param currency: Currency codename (default=constants.CURRENCY_CODENAME_G1)
         """
-        super().__init__(version, currency, [signature])
+        super().__init__(version, currency)
+
         documents_versions = max(
             max([1] + [i.version for i in identities]),
             max([1] + [m.version for m in actives + leavers + joiners]),
@@ -219,9 +214,7 @@ class Block(Document):
         )
         if self.version < documents_versions:
             raise MalformedDocumentError(
-                "Block version is too low : {0} < {1}".format(
-                    self.version, documents_versions
-                )
+                f"Block version is too low: {self.version} < {documents_versions}"
             )
         self.number = number
         self.powmin = powmin
@@ -248,18 +241,31 @@ class Block(Document):
         self.inner_hash = inner_hash
         self.nonce = nonce
 
+        if signing_key is not None:
+            self.sign(signing_key)
+
     @property
-    def blockUID(self) -> BlockUID:
-        return BlockUID(self.number, self.proof_of_work())
+    def block_id(self) -> BlockID:
+        """
+        Return Block UID
+
+        :return:
+        """
+        return BlockID(self.number, self.proof_of_work())
 
     @classmethod
     def from_parsed_json(cls: Type[BlockType], parsed_json_block: dict) -> BlockType:
-        """return a block from the python structure produced when parsing json"""
+        """
+        Return a Block instance from the python dict produced when parsing json
+
+        :param parsed_json_block: Block as a Python dict
+
+        :return:
+        """
         b = parsed_json_block  # alias for readability
         version = b["version"]
         currency = b["currency"]
         arguments = {  # arguments used to create block
-            "version": b["version"],
             "currency": b["currency"],
             "number": b["number"],
             "powmin": b["powMin"],
@@ -285,53 +291,70 @@ class Block(Document):
             "transactions": b["transactions"],
             "inner_hash": b["inner_hash"],
             "nonce": b["nonce"],
-            "signature": b["signature"],
+            "version": b["version"],
         }
         # parameters: Optional[Sequence[str]]
         if arguments["parameters"]:
             arguments["parameters"] = arguments["parameters"].split(":")
         # identities: List[Identity]
         arguments["identities"] = [
-            Identity.from_inline(version, currency, i + "\n")
+            Identity.from_inline(f"{i}\n", version=version, currency=currency)
             for i in arguments["identities"]
         ]
         # joiners: List[Membership]
         arguments["joiners"] = [
-            Membership.from_inline(version, currency, "IN", i + "\n")
+            Membership.from_inline(
+                f"{i}\n", version=version, currency=currency, membership_type="IN"
+            )
             for i in arguments["joiners"]
         ]
         # actives: List[Membership]
         arguments["actives"] = [
-            Membership.from_inline(version, currency, "IN", i + "\n")
+            Membership.from_inline(
+                f"{i}\n", version=version, currency=currency, membership_type="IN"
+            )
             for i in arguments["actives"]
         ]
         # leavers: List[Membership]
         arguments["leavers"] = [
-            Membership.from_inline(version, currency, "OUT", i + "\n")
+            Membership.from_inline(
+                f"{i}\n", version=version, currency=currency, membership_type="OUT"
+            )
             for i in arguments["leavers"]
         ]
         # revokations: List[Revocation]
         arguments["revokations"] = [
-            Revocation.from_inline(version, currency, i + "\n")
+            Revocation.from_inline(f"{i}\n", version=version, currency=currency)
             for i in arguments["revokations"]
         ]
         # certifications: List[Certification]
         arguments["certifications"] = [
             Certification.from_inline(
-                version, currency, arguments["inner_hash"], i + "\n"
+                arguments["inner_hash"], f"{i}\n", version=version, currency=currency
             )
             for i in arguments["certifications"]
         ]
         # transactions: List[Transaction]
         arguments["transactions"] = [
-            Transaction.from_bma_history(currency, i) for i in arguments["transactions"]
+            Transaction.from_bma_history(i, currency=currency)
+            for i in arguments["transactions"]
         ]
 
-        # now that the arguments are ready, return the block
-        return cls(**arguments)
+        block = cls(**arguments)
+
+        # return the block with signature
+        block.signature = b["signature"]
+        return block
 
     @classmethod
     def from_signed_raw(cls: Type[BlockType], signed_raw: str) -> BlockType:
+        """
+        Create Block instance from signed raw format string
+
+        :param signed_raw: Signed raw format string
+
+        :return:
+        """
         lines = signed_raw.splitlines(True)
         n = 0
 
@@ -411,35 +434,45 @@ class Block(Document):
         if Block.re_identities.match(lines[n]) is not None:
             n += 1
             while Block.re_joiners.match(lines[n]) is None:
-                selfcert = Identity.from_inline(version, currency, lines[n])
+                selfcert = Identity.from_inline(
+                    lines[n], version=version, currency=currency
+                )
                 identities.append(selfcert)
                 n += 1
 
         if Block.re_joiners.match(lines[n]):
             n += 1
             while Block.re_actives.match(lines[n]) is None:
-                membership = Membership.from_inline(version, currency, "IN", lines[n])
+                membership = Membership.from_inline(
+                    lines[n], version=version, currency=currency, membership_type="IN"
+                )
                 joiners.append(membership)
                 n += 1
 
         if Block.re_actives.match(lines[n]):
             n += 1
             while Block.re_leavers.match(lines[n]) is None:
-                membership = Membership.from_inline(version, currency, "IN", lines[n])
+                membership = Membership.from_inline(
+                    lines[n], version=version, currency=currency, membership_type="IN"
+                )
                 actives.append(membership)
                 n += 1
 
         if Block.re_leavers.match(lines[n]):
             n += 1
             while Block.re_revoked.match(lines[n]) is None:
-                membership = Membership.from_inline(version, currency, "OUT", lines[n])
+                membership = Membership.from_inline(
+                    lines[n], version=version, currency=currency, membership_type="OUT"
+                )
                 leavers.append(membership)
                 n += 1
 
         if Block.re_revoked.match(lines[n]):
             n += 1
             while Block.re_excluded.match(lines[n]) is None:
-                revokation = Revocation.from_inline(version, currency, lines[n])
+                revokation = Revocation.from_inline(
+                    lines[n], version=version, currency=currency
+                )
                 revoked.append(revokation)
                 n += 1
 
@@ -456,7 +489,7 @@ class Block(Document):
             n += 1
             while Block.re_transactions.match(lines[n]) is None:
                 certification = Certification.from_inline(
-                    version, currency, prev_hash, lines[n]
+                    prev_hash, lines[n], version=version, currency=currency
                 )
                 certifications.append(certification)
                 n += 1
@@ -467,9 +500,7 @@ class Block(Document):
                 tx_lines = ""
                 header_data = Transaction.re_header.match(lines[n])
                 if header_data is None:
-                    raise MalformedDocumentError(
-                        "Compact transaction ({0})".format(lines[n])
-                    )
+                    raise MalformedDocumentError(f"Compact transaction ({lines[n]})")
                 issuers_num = int(header_data.group(2))
                 inputs_num = int(header_data.group(3))
                 unlocks_num = int(header_data.group(4))
@@ -488,7 +519,7 @@ class Block(Document):
                 for index in range(n, tx_max):
                     tx_lines += lines[index]
                 n += tx_max - n
-                transaction = Transaction.from_compact(currency, tx_lines)
+                transaction = Transaction.from_compact(tx_lines, currency=currency)
                 transactions.append(transaction)
 
         inner_hash = Block.parse_field("InnerHash", lines[n])
@@ -499,9 +530,7 @@ class Block(Document):
 
         signature = Block.parse_field("Signature", lines[n])
 
-        return cls(
-            version,
-            currency,
+        block = cls(
             number,
             powmin,
             time,
@@ -526,136 +555,154 @@ class Block(Document):
             transactions,
             inner_hash,
             nonce,
-            signature,
+            version=version,
+            currency=currency,
         )
 
+        # return block with signature
+        block.signature = signature
+        return block
+
     def raw(self) -> str:
-        doc = """Version: {version}
-Type: Block
-Currency: {currency}
-Number: {number}
-PoWMin: {powmin}
-Time: {time}
-MedianTime: {mediantime}
-""".format(
-            version=self.version,
-            currency=self.currency,
-            number=self.number,
-            powmin=self.powmin,
-            time=self.time,
-            mediantime=self.mediantime,
-        )
+        """
+        Return Block in raw format
+
+        :return:
+        """
+        doc = f"Version: {self.version}\n\
+Type: Block\n\
+Currency: {self.currency}\n\
+Number: {self.number}\n\
+PoWMin: {self.powmin}\n\
+Time: {self.time}\n\
+MedianTime: {self.mediantime}\n"
         if self.ud:
-            doc += "UniversalDividend: {0}\n".format(self.ud)
+            doc += f"UniversalDividend: {self.ud}\n"
 
-        doc += "UnitBase: {0}\n".format(self.unit_base)
+        doc += f"UnitBase: {self.unit_base}\n"
 
-        doc += "Issuer: {0}\n".format(self.issuer)
+        doc += f"Issuer: {self.issuer}\n"
 
-        doc += """IssuersFrame: {0}
-IssuersFrameVar: {1}
-DifferentIssuersCount: {2}
-""".format(
-            self.issuers_frame, self.issuers_frame_var, self.different_issuers_count
-        )
+        doc += f"IssuersFrame: {self.issuers_frame}\n\
+IssuersFrameVar: {self.issuers_frame_var}\n\
+DifferentIssuersCount: {self.different_issuers_count}\n"
 
         if self.number == 0 and self.parameters is not None:
             str_params = ":".join([str(p) for p in self.parameters])
-            doc += "Parameters: {0}\n".format(str_params)
+            doc += f"Parameters: {str_params}\n"
         else:
-            doc += "PreviousHash: {0}\n\
-PreviousIssuer: {1}\n".format(
-                self.prev_hash, self.prev_issuer
-            )
+            doc += f"PreviousHash: {self.prev_hash}\n\
+PreviousIssuer: {self.prev_issuer}\n"
 
-        doc += "MembersCount: {0}\n".format(self.members_count)
+        doc += f"MembersCount: {self.members_count}\n"
 
         doc += "Identities:\n"
         for identity in self.identities:
-            doc += "{0}\n".format(identity.inline())
+            doc += f"{identity.inline()}\n"
 
         doc += "Joiners:\n"
         for joiner in self.joiners:
-            doc += "{0}\n".format(joiner.inline())
+            doc += f"{joiner.inline()}\n"
 
         doc += "Actives:\n"
         for active in self.actives:
-            doc += "{0}\n".format(active.inline())
+            doc += f"{active.inline()}\n"
 
         doc += "Leavers:\n"
         for leaver in self.leavers:
-            doc += "{0}\n".format(leaver.inline())
+            doc += f"{leaver.inline()}\n"
 
         doc += "Revoked:\n"
         for revokation in self.revoked:
-            doc += "{0}\n".format(revokation.inline())
+            doc += f"{revokation.inline()}\n"
 
         doc += "Excluded:\n"
         for exclude in self.excluded:
-            doc += "{0}\n".format(exclude)
+            doc += f"{exclude}\n"
 
         doc += "Certifications:\n"
         for cert in self.certifications:
-            doc += "{0}\n".format(cert.inline())
+            doc += f"{cert.inline()}\n"
 
         doc += "Transactions:\n"
         for transaction in self.transactions:
-            doc += "{0}".format(transaction.compact())
+            doc += f"{transaction.compact()}"
 
-        doc += "InnerHash: {0}\n".format(self.inner_hash)
+        doc += f"InnerHash: {self.inner_hash}\n"
 
-        doc += "Nonce: {0}\n".format(self.nonce)
+        doc += f"Nonce: {self.nonce}\n"
 
         return doc
 
     def proof_of_work(self) -> str:
-        doc_str = """InnerHash: {inner_hash}
-Nonce: {nonce}
-{signature}
-""".format(
-            inner_hash=self.inner_hash, nonce=self.nonce, signature=self.signatures[0]
+        """
+        Return Proof of Work hash for the Block
+
+        :return:
+        """
+        doc_str = (
+            f"InnerHash: {self.inner_hash}\nNonce: {self.nonce}\n{self.signature}\n"
         )
         return hashlib.sha256(doc_str.encode("ascii")).hexdigest().upper()
 
     def computed_inner_hash(self) -> str:
+        """
+        Return inner hash of the Block
+
+        :return:
+        """
         doc = self.raw()
         inner_doc = "\n".join(doc.split("\n")[:-3]) + "\n"
         return hashlib.sha256(inner_doc.encode("ascii")).hexdigest().upper()
 
-    def sign(self, keys):
+    def sign(self, key: SigningKey) -> None:
         """
-        Sign the current document.
-        Warning : current signatures will be replaced with the new ones.
+        Sign the current document with key
+
+        :param key: Libnacl SigningKey instance
+
+        :return:
         """
-        key = keys[0]
-        signed = "InnerHash: {inner_hash}\nNonce: {nonce}\n".format(
-            inner_hash=self.inner_hash,
-            nonce=self.nonce,
-        )
-        signing = base64.b64encode(key.signature(bytes(signed, "ascii")))
-        self.signatures = [signing.decode("ascii")]
+        string_to_sign = f"InnerHash: {self.inner_hash}\nNonce: {self.nonce}\n"
+        signature = base64.b64encode(key.signature(bytes(string_to_sign, "ascii")))
+        self.signature = signature.decode("ascii")
 
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, Block):
             return NotImplemented
-        return self.blockUID == other.blockUID
+        return self.block_id == other.block_id
 
     def __lt__(self, other: object) -> bool:
         if not isinstance(other, Block):
             return False
-        return self.blockUID < other.blockUID
+        return self.block_id < other.block_id
 
     def __gt__(self, other: object) -> bool:
         if not isinstance(other, Block):
             return False
-        return self.blockUID > other.blockUID
+        return self.block_id > other.block_id
 
     def __le__(self, other: object) -> bool:
         if not isinstance(other, Block):
             return False
-        return self.blockUID <= other.blockUID
+        return self.block_id <= other.block_id
 
     def __ge__(self, other: object) -> bool:
         if not isinstance(other, Block):
             return False
-        return self.blockUID >= other.blockUID
+        return self.block_id >= other.block_id
+
+    def check_signature(self, pubkey: str):
+        """
+        Check if the signature is from pubkey
+
+        :param pubkey: Base58 public key
+
+        :return:
+        """
+        if self.signature is None:
+            raise Exception("Signature is None, can not verify signature")
+
+        content_to_verify = f"InnerHash: {self.inner_hash}\nNonce: {self.nonce}\n"
+        verifying_key = VerifyingKey(pubkey)
+        return verifying_key.check_signature(content_to_verify, self.signature)
