@@ -1,34 +1,34 @@
-"""
-Copyright  2014-2021 Vincent Texier <vit@free.fr>
-
-DuniterPy is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-DuniterPy is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# Copyright  2014-2022 Vincent Texier <vit@free.fr>
+#
+# DuniterPy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# DuniterPy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import re
+
 import attr
 
-from ..block_uid import BlockUID
-from ..document import MalformedDocumentError
 from ...constants import (
-    WS2P_PUBLIC_PREFIX_REGEX,
-    WS2P_PRIVATE_PREFIX_REGEX,
-    WS2P_HEAD_REGEX,
+    BLOCK_ID_REGEX,
     PUBKEY_REGEX,
     SIGNATURE_REGEX,
+    WS2P_HEAD_REGEX,
+    WS2P_PRIVATE_PREFIX_REGEX,
+    WS2P_PUBLIC_PREFIX_REGEX,
     WS2PID_REGEX,
-    BLOCK_UID_REGEX,
 )
+from ...key import VerifyingKey
+from ..block_id import BlockID
+from ..document import MalformedDocumentError
 
 
 @attr.s()
@@ -37,9 +37,7 @@ class API:
     public = attr.ib(type=str)
 
     re_inline = re.compile(
-        "WS2P({ws2p_private})?({ws2p_public})?".format(
-            ws2p_private=WS2P_PRIVATE_PREFIX_REGEX, ws2p_public=WS2P_PUBLIC_PREFIX_REGEX
-        )
+        f"WS2P({WS2P_PRIVATE_PREFIX_REGEX})?({WS2P_PUBLIC_PREFIX_REGEX})?"
     )
 
     @classmethod
@@ -52,7 +50,7 @@ class API:
         return cls(private, public)
 
     def __str__(self) -> str:
-        return "WS2P" + self.private + self.public
+        return f"WS2P{self.private}{self.public}"
 
 
 @attr.s()
@@ -74,7 +72,7 @@ class Head:
             raise MalformedDocumentError("Head") from AttributeError
 
     def __str__(self) -> str:
-        return "HEAD" if self.version == 0 else "HEAD:{}".format(str(self.version))
+        return "HEAD" if self.version == 0 else f"HEAD:{str(self.version)}"
 
 
 @attr.s()
@@ -83,16 +81,11 @@ class HeadV0(Head):
     api = attr.ib(type=API)
     head = attr.ib(type=Head)
     pubkey = attr.ib(type=str)
-    blockstamp = attr.ib(type=BlockUID)
+    block_id = attr.ib(type=BlockID)
 
     re_inline = re.compile(
-        "^(WS2P(?:{ws2p_private})?(?:{ws2p_public})?):({head}):({pubkey}):({blockstamp})(?::)?(.*)".format(
-            ws2p_private=WS2P_PRIVATE_PREFIX_REGEX,
-            ws2p_public=WS2P_PUBLIC_PREFIX_REGEX,
-            head=WS2P_HEAD_REGEX,
-            pubkey=PUBKEY_REGEX,
-            blockstamp=BLOCK_UID_REGEX,
-        )
+        f"^(WS2P(?:{WS2P_PRIVATE_PREFIX_REGEX})?(?:{WS2P_PUBLIC_PREFIX_REGEX})?):\
+({WS2P_HEAD_REGEX}):({PUBKEY_REGEX}):({BLOCK_ID_REGEX})(?::)?(.*)"
     )
 
     re_signature = re.compile(SIGNATURE_REGEX)
@@ -106,9 +99,9 @@ class HeadV0(Head):
             api = API.from_inline(data.group(1))
             head = Head.from_inline(data.group(2), "")
             pubkey = data.group(3)
-            blockstamp = BlockUID.from_str(data.group(4))
+            block_id = BlockID.from_str(data.group(4))
             offload = data.group(5)
-            return cls(head.version, signature, api, head, pubkey, blockstamp), offload
+            return cls(head.version, signature, api, head, pubkey, block_id), offload
         except AttributeError:
             raise MalformedDocumentError("HeadV0") from AttributeError
 
@@ -125,7 +118,18 @@ class HeadV0(Head):
                 ),
             )
         )
-        return "{0}:{1}".format(str(self.api), ":".join(values))
+        return f'{str(self.api)}:{":".join(values)}'
+
+    def check_signature(self, pubkey: str) -> bool:
+        """
+        Check if Head signature is from head pubkey
+
+        :param pubkey: Pubkey to check signature upon
+        :return:
+        """
+        verifying_key = VerifyingKey(pubkey)
+
+        return verifying_key.check_signature(self.inline(), self.signature)
 
 
 @attr.s()
@@ -163,7 +167,7 @@ class HeadV1(HeadV0):
                     v0.api,
                     v0.head,
                     v0.pubkey,
-                    v0.blockstamp,
+                    v0.block_id,
                     ws2pid,
                     software,
                     software_version,
@@ -202,7 +206,7 @@ class HeadV2(HeadV1):
                     v1.api,
                     v1.head,
                     v1.pubkey,
-                    v1.blockstamp,
+                    v1.block_id,
                     v1.ws2pid,
                     v1.software,
                     v1.software_version,
