@@ -1,27 +1,28 @@
-"""
-Copyright  2014-2021 Vincent Texier <vit@free.fr>
-
-DuniterPy is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-DuniterPy is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# Copyright  2014-2022 Vincent Texier <vit@free.fr>
+#
+# DuniterPy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# DuniterPy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import base64
+import re
+from typing import Any, Dict, List, Optional
+
 import libnacl
 from libnacl.version import version as libnacl_version
-import re
-from typing import Optional, List, Dict, Any
 
-from duniterpy.key import SigningKey, PublicKey, VerifyingKey
+from .encryption_key import PublicKey
+from .signing_key import SigningKey
+from .verifying_key import VerifyingKey
 
 # Headers constants
 BEGIN_MESSAGE_HEADER = "-----BEGIN DUNITER MESSAGE-----"
@@ -32,7 +33,7 @@ HEADER_PREFIX = "-----"
 DASH_ESCAPE_PREFIX = "\x2D\x20"
 
 # Version field value
-VERSION_FIELD_VALUE = "Python Libnacl " + libnacl_version
+VERSION_FIELD_VALUE = f"Python Libnacl {libnacl_version}"
 
 # Parser cursor status
 ON_MESSAGE_FIELDS = 1
@@ -107,25 +108,18 @@ class AsciiArmor:
         message = message.rstrip("\n\r") + "\n"
 
         # create block with headers
-        ascii_armor_block = """{begin_message_header}
-""".format(
-            begin_message_header=BEGIN_MESSAGE_HEADER
-        )
+        ascii_armor_block = f"{BEGIN_MESSAGE_HEADER}\n"
 
         # if encrypted message...
         if pubkey:
             # add encrypted message fields
-            ascii_armor_block += """{version_field}
-""".format(
-                version_field=AsciiArmor._get_version_field()
-            )
+            version_field = AsciiArmor._get_version_field()
+            ascii_armor_block += f"{version_field}\n"
 
         # add message comment if specified
         if message_comment:
-            ascii_armor_block += """{comment_field}
-""".format(
-                comment_field=AsciiArmor._get_comment_field(message_comment)
-            )
+            comment_field = AsciiArmor._get_comment_field(message_comment)
+            ascii_armor_block += f"{comment_field}\n"
 
         # blank line separator
         ascii_armor_block += "\n"
@@ -136,10 +130,7 @@ class AsciiArmor:
             base64_encrypted_message = base64.b64encode(
                 pubkey_instance.encrypt_seal(message)
             )  # type: bytes
-            ascii_armor_block += """{base64_encrypted_message}
-""".format(
-                base64_encrypted_message=base64_encrypted_message.decode("utf-8")
-            )
+            ascii_armor_block += f'{base64_encrypted_message.decode("utf-8")}\n'
         else:
             # remove trailing spaces
             message = AsciiArmor._remove_trailing_spaces(message)
@@ -170,7 +161,7 @@ class AsciiArmor:
         :param text: Text to clean up
         :return:
         """
-        clean_text = str()
+        clean_text = ""
 
         for line in text.splitlines(True):
             # remove trailing spaces (0x20) and tabs (0x09)
@@ -186,7 +177,7 @@ class AsciiArmor:
         :param text: Text to dash-escape
         :return:
         """
-        dash_escaped_text = str()
+        dash_escaped_text = ""
 
         for line in text.splitlines(True):
             # add dash '-' (0x2D) and space ' ' (0x20) as prefix
@@ -202,7 +193,7 @@ class AsciiArmor:
         :param dash_escaped_line: Dash escaped text line
         :return:
         """
-        text = str()
+        text = ""
         regex_dash_escape_prefix = re.compile("^" + DASH_ESCAPE_PREFIX)
         # if prefixed by a dash escape prefix...
         if regex_dash_escape_prefix.match(dash_escaped_line):
@@ -218,7 +209,7 @@ class AsciiArmor:
 
         :return:
         """
-        return "Version: {version}".format(version=VERSION_FIELD_VALUE)
+        return f"Version: {VERSION_FIELD_VALUE}"
 
     @staticmethod
     def _get_comment_field(comment: str) -> str:
@@ -228,7 +219,7 @@ class AsciiArmor:
         :param comment: Comment text
         :return:
         """
-        return "Comment: {comment}".format(comment=comment)
+        return f"Comment: {comment}"
 
     @staticmethod
     def _get_signature_block(
@@ -248,27 +239,16 @@ class AsciiArmor:
         """
         base64_signature = base64.b64encode(signing_key.signature(message))
 
-        block = """{begin_signature_header}
-{version_field}
-""".format(
-            begin_signature_header=BEGIN_SIGNATURE_HEADER,
-            version_field=AsciiArmor._get_version_field(),
-        )
+        version_field = AsciiArmor._get_version_field()
+        block = f"{BEGIN_SIGNATURE_HEADER}\n{version_field}\n"
 
         # add message comment if specified
         if comment:
-            block += """{comment_field}
-""".format(
-                comment_field=AsciiArmor._get_comment_field(comment)
-            )
+            comment_field = AsciiArmor._get_comment_field(comment)
+            block += f"{comment_field}\n"
 
         # blank line separator
-        block += "\n"
-
-        block += """{base64_signature}
-""".format(
-            base64_signature=base64_signature.decode("utf-8")
-        )
+        block += f'\n{base64_signature.decode("utf-8")}\n'
 
         if close_block:
             block += END_SIGNATURE_HEADER
