@@ -1,28 +1,26 @@
-"""
-Copyright  2014-2021 Vincent Texier <vit@free.fr>
+# Copyright  2014-2022 Vincent Texier <vit@free.fr>
+#
+# DuniterPy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# DuniterPy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-DuniterPy is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-DuniterPy is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
-
-from typing import Union, Optional
+from hashlib import scrypt
+from typing import Optional, Union
 
 import libnacl.public
-from hashlib import scrypt
 
-from .scrypt_params import ScryptParams
-from .base58 import Base58Encoder
 from ..tools import ensure_bytes
+from .base58 import Base58Encoder
+from .scrypt_params import ScryptParams
 
 
 class SecretKey(libnacl.public.SecretKey):
