@@ -1,37 +1,35 @@
-"""
-Copyright  2014-2021 Vincent Texier <vit@free.fr>
-
-DuniterPy is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-DuniterPy is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# Copyright  2014-2022 Vincent Texier <vit@free.fr>
+#
+# DuniterPy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# DuniterPy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import base64
 import re
-from typing import Optional, Union, TypeVar, Type
+from hashlib import scrypt, sha256
+from typing import Optional, Type, TypeVar, Union
 
 import libnacl.sign
 import pyaes
 from libnacl.utils import load_key
-from hashlib import scrypt
 
-from .scrypt_params import ScryptParams
-from .base58 import Base58Encoder
 from ..tools import (
+    convert_seed_to_seedhex,
+    convert_seedhex_to_seed,
     ensure_bytes,
     xor_bytes,
-    convert_seedhex_to_seed,
-    convert_seed_to_seedhex,
 )
+from .base58 import Base58Encoder
+from .scrypt_params import ScryptParams
 
 # required to type hint cls in classmethod
 SigningKeyType = TypeVar("SigningKeyType", bound="SigningKey")
@@ -91,7 +89,7 @@ class SigningKey(libnacl.sign.Signer):
         :return:
         """
         # capture credentials from file
-        with open(path, "r") as fh:
+        with open(path, encoding="utf-8") as fh:
             lines = fh.readlines()
             assert len(lines) > 1
             salt = lines[0].strip()
@@ -106,7 +104,7 @@ class SigningKey(libnacl.sign.Signer):
         :param path: Authentication file path
         """
         seedhex = convert_seed_to_seedhex(self.seed)
-        with open(path, "w") as fh:
+        with open(path, "w", encoding="utf-8") as fh:
             fh.write(seedhex)
 
     @staticmethod
@@ -116,7 +114,7 @@ class SigningKey(libnacl.sign.Signer):
 
         :param str path: Hexadecimal seed file path
         """
-        with open(path, "r") as fh:
+        with open(path, encoding="utf-8") as fh:
             seedhex = fh.read()
         return SigningKey.from_seedhex(seedhex)
 
@@ -176,7 +174,7 @@ class SigningKey(libnacl.sign.Signer):
 
         :param path: Path to WIF file
         """
-        with open(path, "r") as fh:
+        with open(path, encoding="utf-8") as fh:
             pubsec_content = fh.read()
 
         # line patterns
@@ -212,20 +210,14 @@ class SigningKey(libnacl.sign.Signer):
 
         # base58 encode keys
         base58_signing_key = Base58Encoder.encode(self.sk)
-        base58_public_key = self.pubkey
 
         # save file
-        with open(path, "w") as fh:
-            fh.write(
-                """Type: PubSec
-Version: {version}
-pub: {pubkey}
-sec: {signkey}""".format(
-                    version=version,
-                    pubkey=base58_public_key,
-                    signkey=base58_signing_key,
-                )
-            )
+        content = f"Type: PubSec\n\
+Version: {version}\n\
+pub: {self.pubkey}\n\
+sec: {base58_signing_key}"
+        with open(path, "w", encoding="utf-8") as fh:
+            fh.write(content)
 
     @staticmethod
     def from_wif_or_ewif_file(
@@ -237,7 +229,7 @@ sec: {signkey}""".format(
         :param path: Path to WIF of EWIF file
         :param password: Password needed for EWIF file
         """
-        with open(path, "r") as fh:
+        with open(path, encoding="utf-8") as fh:
             wif_content = fh.read()
 
         # check data field
@@ -280,7 +272,7 @@ sec: {signkey}""".format(
 
         :param path: Path to WIF file
         """
-        with open(path, "r") as fh:
+        with open(path, encoding="utf-8") as fh:
             wif_content = fh.read()
 
         # check data field
@@ -341,14 +333,11 @@ sec: {signkey}""".format(
         # base58 encode key and checksum
         wif_key = Base58Encoder.encode(seed_fi + checksum)
 
-        with open(path, "w") as fh:
-            fh.write(
-                """Type: WIF
-Version: {version}
-Data: {data}""".format(
-                    version=version, data=wif_key
-                )
-            )
+        content = f"Type: WIF\n\
+Version: {version}\n\
+Data: {wif_key}"
+        with open(path, "w", encoding="utf-8") as fh:
+            fh.write(content)
 
     @staticmethod
     def from_ewif_file(path: str, password: str) -> SigningKeyType:
@@ -358,7 +347,7 @@ Data: {data}""".format(
         :param path: Path to EWIF file
         :param password: Password of the encrypted seed
         """
-        with open(path, "r") as fh:
+        with open(path, encoding="utf-8") as fh:
             wif_content = fh.read()
 
         # check data field
@@ -474,14 +463,11 @@ Data: {data}""".format(
         ewif_key = Base58Encoder.encode(seed_bytes + checksum)
 
         # save file
-        with open(path, "w") as fh:
-            fh.write(
-                """Type: EWIF
-Version: {version}
-Data: {data}""".format(
-                    version=version, data=ewif_key
-                )
-            )
+        content = f"Type: EWIF\n\
+Version: {version}\n\
+Data: {ewif_key}"
+        with open(path, "w", encoding="utf-8") as fh:
+            fh.write(content)
 
     @classmethod
     def from_ssb_file(cls: Type[SigningKeyType], path: str) -> SigningKeyType:
@@ -490,7 +476,7 @@ Data: {data}""".format(
 
         :param path: Path to Scuttlebutt secret file
         """
-        with open(path, "r") as fh:
+        with open(path, encoding="utf-8") as fh:
             ssb_content = fh.read()
 
         # check data field
@@ -509,3 +495,31 @@ Data: {data}""".format(
         seed = bytes(base64.b64decode(secret)[0:32])
 
         return cls(seed)
+
+    @classmethod
+    def from_dubp_mnemonic(
+        cls, mnemonic: str, scrypt_params: ScryptParams = None
+    ) -> SigningKeyType:
+        """
+        Generate key pair instance from a DUBP mnemonic passphrase
+
+        See https://git.duniter.org/documents/rfcs/blob/dubp-mnemonic/rfc/0014_Dubp_Mnemonic.md
+
+        :param mnemonic: Passphrase generated from a mnemonic algorithm
+        :param scrypt_params: ScryptParams instance (default=None)
+        :return:
+        """
+        if scrypt_params is None:
+            scrypt_params = ScryptParams()
+
+        _password = mnemonic.encode("utf-8")  # type: bytes
+        _salt = sha256(b"dubp" + _password).digest()  # type: bytes
+        _seed = scrypt(
+            password=_password,
+            salt=_salt,
+            n=scrypt_params.N,  # 4096
+            r=scrypt_params.r,  # 16
+            p=scrypt_params.p,  # 1
+            dklen=scrypt_params.seed_length,  # 32
+        )  # type: bytes
+        return cls(_seed)
