@@ -1,32 +1,35 @@
-"""
-Copyright  2014-2021 Vincent Texier <vit@free.fr>
+# Copyright  2014-2022 Vincent Texier <vit@free.fr>
+#
+# DuniterPy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# DuniterPy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-DuniterPy is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-DuniterPy is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
 import unittest
+
 import pypeg2
-from duniterpy.grammars import output
-from duniterpy.documents import BlockUID
+
+from duniterpy.constants import G1_TEST_CURRENCY_CODENAME
+from duniterpy.documents import BlockID
 from duniterpy.documents.transaction import (
-    Transaction,
-    reduce_base,
-    SimpleTransaction,
     InputSource,
     OutputSource,
-    Unlock,
     SIGParameter,
+    SimpleTransaction,
+    Transaction,
+    Unlock,
+    reduce_base,
 )
+from duniterpy.grammars import output
+from duniterpy.key import SigningKey
 
 compact_change = """TX:10:1:1:1:1:1:0
 13410-000041DF0CCA173F09B5FBA48F619D4BC934F12ADF1D0B798639EB2149C4A8CC
@@ -141,6 +144,37 @@ Comment: -----@@@----- (why not this comment?)
 2XiBDpuUdu6zCPWGzHXXy8c4ATSscfFQG9DjmqMZUxDZVt1Dp4m2N5oHYVUfoPdrU9SLk4qxi65RNrfCVnvQtQJk
 """
 
+# curl https://g1-test.duniter.org/blockchain/block/551979
+g1_test_tx_multi_sig_compact = """TX:10:6:6:6:1:1:0
+551977-0001108622DBA433872C547D342B033193B16E7C60C478C264DC4F1B5F36AC74
+CLxmBmQDxT4r5thHhQHQk3LVJZoerpMArkTGSfkeexkg
+5gHpK5sfijCkDDxSfiCWa3QxUsXnd38ZX3Frn7qaG42b
+6bcCUCMmTVQjqc3VGuA1xQv3ETKDV1JJDM5GGCfn58A4
+GM6jrbZSgd2WaE8LY9WP4V2v1MDpxbqMVUBBxdJswz1s
+AfkSYeiqAVHPqaKHhw52fR4rgC4QGaLGCTBP9jHfeDMQ
+8tqeZi9pKV8VFjY7CNce4FbHvv76f7y3uKcyJ865pZzB
+250:2:T:9E5AB3FAED93C9C65D4D0C16C1387592A7B9E1AE7285DB6C687FFE6AAF542CF2:0
+250:2:T:9E5AB3FAED93C9C65D4D0C16C1387592A7B9E1AE7285DB6C687FFE6AAF542CF2:1
+250:2:T:9E5AB3FAED93C9C65D4D0C16C1387592A7B9E1AE7285DB6C687FFE6AAF542CF2:2
+250:2:T:9E5AB3FAED93C9C65D4D0C16C1387592A7B9E1AE7285DB6C687FFE6AAF542CF2:3
+250:2:T:9E5AB3FAED93C9C65D4D0C16C1387592A7B9E1AE7285DB6C687FFE6AAF542CF2:4
+250:2:T:9E5AB3FAED93C9C65D4D0C16C1387592A7B9E1AE7285DB6C687FFE6AAF542CF2:5
+0:SIG(0)
+1:SIG(1)
+2:SIG(2)
+3:SIG(3)
+4:SIG(4)
+5:SIG(5)
+1500:2:SIG(AhRMHUxMPXSeG7qXZrE6qCdjwK9p2bu5Eqei7xAWVEDK)
+G1Don
+ukn60jVqcaR5ZJdGOW6aZapUSyCpRQkQWtKBcndTXtLJP5zs26bAl5XfASDCqivecuWyNedU1y2sAhA/OqcADg==
+kcybaONx12s8ZJuRVVyGrRXBgYlnUr7CGOdm0/75F9yNpQ8U/8lzyPMKugbJ6txPQ7/TBf6cS6XMTwoZH8PPBQ==
+UM9eT2iW0br42yBznimxjuVJJNFtGICUwNHspPsYOraWqk4prEte5lh9b88Q/HtDZN9o4qZzXuoVYOwbvNfgBg==
+LSqXaG4MylpG39r4OuxGbTXm/cLXOvCv60BarTR9QABW7uKjgIra5+EELRxW8Ueg9QAimfNBBIHy/ZVO+saHAQ==
+wI7cEq8nhZbi/uDOx4DawqQNoVlqpWQ1N2XgFwz0AdTubgJc8/JxDOG+xDYk8P+9m4oni3NjC58jNWzOqd+TDA==
+KjAnwP8gDyHr228py7xAFvaYoK2NQ6CHbZADK0m1qx0pxViB18m5Qs9MVAFjYTDQop9qfSie8nX/9NLdW09jBQ==
+"""
+
 input_source_str = (
     "30:0:T:6991C993631BED4733972ED7538E41CCC33660F554E3C51963E2A0AC4D6453D3:2"
 )
@@ -150,7 +184,7 @@ output_source_str = "460:0:SIG(8kXygUHh1vLjmcRzXVM86t38EL8dfFJgfBeHmkaWLamu)"
 
 class TestTransaction(unittest.TestCase):
     def test_fromcompact(self):
-        tx = Transaction.from_compact("zeta_brousouf", tx_compact)
+        tx = Transaction.from_compact(tx_compact, "zeta_brousouf")
         self.assertEqual(tx.version, 10)
         self.assertEqual(tx.currency, "zeta_brousouf")
         self.assertEqual(len(tx.issuers), 3)
@@ -267,9 +301,9 @@ class TestTransaction(unittest.TestCase):
 
         self.assertEqual(tx.version, 10)
         self.assertEqual(tx.currency, "beta_brousouf")
-        self.assertEqual(tx.blockstamp.number, 32)
+        self.assertEqual(tx.block_id.number, 32)
         self.assertEqual(
-            tx.blockstamp.sha_hash, "DB30D958EE5CB75186972286ED3F4686B8A1C2CD"
+            tx.block_id.sha_hash, "DB30D958EE5CB75186972286ED3F4686B8A1C2CD"
         )
         self.assertEqual(len(tx.issuers), 3)
         self.assertEqual(len(tx.inputs), 6)
@@ -366,7 +400,7 @@ class TestTransaction(unittest.TestCase):
         )
 
     def test_compact_change(self):
-        tx = Transaction.from_compact("gtest", compact_change)
+        tx = Transaction.from_compact(compact_change, "gtest")
         rendered_tx = tx.signed_raw()
         self.assertEqual(tx_from_compact_change, rendered_tx)
         Transaction.from_signed_raw(rendered_tx)
@@ -386,10 +420,10 @@ class TestTransaction(unittest.TestCase):
         self.assertEqual(computed[1], 5)
 
     def test_is_simple(self):
-        tx = Transaction.from_compact("zeta_brousouf", simple_tx_compact)
+        tx = Transaction.from_compact(simple_tx_compact, "zeta_brousouf")
         self.assertTrue(SimpleTransaction.is_simple(tx))
 
-        tx = Transaction.from_compact("zeta_brousouf", tx_compact)
+        tx = Transaction.from_compact(tx_compact, "zeta_brousouf")
         self.assertFalse(SimpleTransaction.is_simple(tx))
 
     def test_inputsource_from_inline(self):
@@ -431,9 +465,7 @@ class TestTransaction(unittest.TestCase):
 
     def test_transaction_document_generation(self):
         transaction = Transaction(
-            version=10,
-            currency="gtest",
-            blockstamp=BlockUID(
+            block_id=BlockID(
                 8979, "000041DF0CCA173F09B5FBA48F619D4BC934F12ADF1D0B798639EB2149C4A8CC"
             ),
             locktime=0,
@@ -442,7 +474,7 @@ class TestTransaction(unittest.TestCase):
             unlocks=[Unlock(index=0, parameters=[SIGParameter(0)])],
             outputs=[OutputSource.from_inline(output_source_str)],
             comment="",
-            signatures=[],
+            currency="gtest",
         )
         self.assertTrue(transaction.time is None)
         self.assertTrue(transaction.currency == "gtest")
@@ -452,3 +484,71 @@ class TestTransaction(unittest.TestCase):
         unlock1 = Unlock(0, [SIGParameter(0)])
         unlock2 = Unlock.from_inline("0:SIG(0)")
         self.assertEqual(unlock1, unlock2)
+
+    def test_check_signature(self):
+        # create 3 wallets
+        signing_key = SigningKey.from_credentials("1", "1")
+
+        issuer = signing_key.pubkey
+
+        transaction = Transaction(
+            block_id=BlockID(
+                8979, "000041DF0CCA173F09B5FBA48F619D4BC934F12ADF1D0B798639EB2149C4A8CC"
+            ),
+            locktime=0,
+            issuers=[issuer],
+            inputs=[InputSource.from_inline(input_source_str)],
+            unlocks=[Unlock(index=0, parameters=[SIGParameter(0)])],
+            outputs=[OutputSource.from_inline(output_source_str)],
+            comment="",
+            currency=G1_TEST_CURRENCY_CODENAME,
+        )
+
+        # multi-signature on the transaction
+        transaction.sign(signing_key)
+
+        self.assertTrue(transaction.check_signature(issuer))
+
+    def test_check_signatures(self):
+        # create 3 wallets
+        signing_key_01 = SigningKey.from_credentials("1", "1")
+        signing_key_02 = SigningKey.from_credentials("2", "2")
+        signing_key_03 = SigningKey.from_credentials("3", "3")
+
+        issuers = [
+            signing_key_01.pubkey,
+            signing_key_02.pubkey,
+            signing_key_03.pubkey,
+        ]
+
+        transaction = Transaction(
+            block_id=BlockID(
+                8979, "000041DF0CCA173F09B5FBA48F619D4BC934F12ADF1D0B798639EB2149C4A8CC"
+            ),
+            locktime=0,
+            issuers=issuers,
+            inputs=[InputSource.from_inline(input_source_str)],
+            unlocks=[Unlock(index=0, parameters=[SIGParameter(0)])],
+            outputs=[OutputSource.from_inline(output_source_str)],
+            comment="",
+            currency=G1_TEST_CURRENCY_CODENAME,
+        )
+
+        # multi-signature on the transaction
+        transaction.multi_sign(
+            [
+                signing_key_01,
+                signing_key_02,
+                signing_key_03,
+            ]
+        )
+
+        self.assertTrue(transaction.check_signatures(issuers))
+
+    def test_g1_test_tx_check_signatures(self):
+        tx_multi = Transaction.from_compact(
+            g1_test_tx_multi_sig_compact,
+            G1_TEST_CURRENCY_CODENAME,
+        )
+        # multi signatures check for the pubkeys on the signatures
+        self.assertTrue(tx_multi.check_signatures(tx_multi.issuers))
